import { ApiObject, ApiObjectMetadataDefinition } from 'cdk8s';
import { Construct } from 'constructs';
import { Resource, ResourceProps } from './base';
import { Container, ContainerProps } from './container';
import * as k8s from './imports/k8s';
import { RestartPolicy, IPodTemplate, PodTemplateProps } from './pod';
import { Service } from './service';
import { IServiceAccount } from './service-account';
import { Volume } from './volume';
/**
 * (experimental) Controls how pods are created during initial scale up, when replacing pods on nodes, or when scaling down.
 *
 * The default policy is `OrderedReady`, where pods are created in increasing order
 * (pod-0, then pod-1, etc) and the controller will wait until each pod is ready before
 * continuing. When scaling down, the pods are removed in the opposite order.
 *
 * The alternative policy is `Parallel` which will create pods in parallel to match the
 * desired scale without waiting, and on scale down will delete all pods at once.
 *
 * @experimental
 */
export declare enum PodManagementPolicy {
    /**
     * @experimental
     */
    ORDERED_READY = "OrderedReady",
    /**
     * @experimental
     */
    PARALLEL = "Parallel"
}
/**
 * (experimental) Properties for initialization of `StatefulSet`.
 *
 * @experimental
 */
export interface StatefulSetProps extends ResourceProps, PodTemplateProps {
    /**
     * (experimental) Service to associate with the statefulset.
     *
     * @experimental
     */
    readonly service: Service;
    /**
     * (experimental) Number of desired pods.
     *
     * @default 1
     * @experimental
     */
    readonly replicas?: number;
    /**
     * (experimental) Automatically allocates a pod selector for this statefulset.
     *
     * If this is set to `false` you must define your selector through
     * `statefulset.podMetadata.addLabel()` and `statefulset.selectByLabel()`.
     *
     * @default true
     * @experimental
     */
    readonly defaultSelector?: boolean;
    /**
     * (experimental) Pod management policy to use for this statefulset.
     *
     * @default PodManagementPolicy.ORDERED_READY
     * @experimental
     */
    readonly podManagementPolicy?: PodManagementPolicy;
}
/**
 * (experimental) StatefulSet is the workload API object used to manage stateful applications.
 *
 * Manages the deployment and scaling of a set of Pods, and provides guarantees
 * about the ordering and uniqueness of these Pods.
 *
 * Like a Deployment, a StatefulSet manages Pods that are based on an identical
 * container spec. Unlike a Deployment, a StatefulSet maintains a sticky identity
 * for each of their Pods. These pods are created from the same spec, but are not
 * interchangeable: each has a persistent identifier that it maintains across any
 * rescheduling.
 *
 * If you want to use storage volumes to provide persistence for your workload, you
 * can use a StatefulSet as part of the solution. Although individual Pods in a StatefulSet
 * are susceptible to failure, the persistent Pod identifiers make it easier to match existing
 * volumes to the new Pods that replace any that have failed.
 *
 * Using StatefulSets
 * ------------------
 * StatefulSets are valuable for applications that require one or more of the following.
 *
 * - Stable, unique network identifiers.
 * - Stable, persistent storage.
 * - Ordered, graceful deployment and scaling.
 * - Ordered, automated rolling updates.
 *
 * @experimental
 */
export declare class StatefulSet extends Resource implements IPodTemplate {
    /**
     * (experimental) Number of desired pods.
     *
     * @experimental
     */
    readonly replicas: number;
    /**
     * (experimental) Management policy to use for the set.
     *
     * @experimental
     */
    readonly podManagementPolicy: PodManagementPolicy;
    /**
     * (experimental) The underlying cdk8s API object.
     *
     * @see base.Resource.apiObject
     * @experimental
     */
    protected readonly apiObject: ApiObject;
    private readonly _podTemplate;
    private readonly _labelSelector;
    private readonly _service;
    /**
     * @experimental
     */
    constructor(scope: Construct, id: string, props: StatefulSetProps);
    /**
     * (experimental) Provides read/write access to the underlying pod metadata of the resource.
     *
     * @experimental
     */
    get podMetadata(): ApiObjectMetadataDefinition;
    /**
     * (experimental) The labels this statefulset will match against in order to select pods.
     *
     * Returns a a copy. Use `selectByLabel()` to add labels.
     *
     * @experimental
     */
    get labelSelector(): Record<string, string>;
    /**
     * (experimental) The containers belonging to the pod.
     *
     * Use `addContainer` to add containers.
     *
     * @experimental
     */
    get containers(): Container[];
    /**
     * (experimental) The volumes associated with this pod.
     *
     * Use `addVolume` to add volumes.
     *
     * @experimental
     */
    get volumes(): Volume[];
    /**
     * (experimental) Restart policy for all containers within the pod.
     *
     * @experimental
     */
    get restartPolicy(): RestartPolicy | undefined;
    /**
     * (experimental) The service account used to run this pod.
     *
     * @experimental
     */
    get serviceAccount(): IServiceAccount | undefined;
    /**
     * (experimental) Configure a label selector to this deployment.
     *
     * Pods that have the label will be selected by deployments configured with this spec.
     *
     * @param key - The label key.
     * @param value - The label value.
     * @experimental
     */
    selectByLabel(key: string, value: string): void;
    /**
     * (experimental) Add a container to the pod.
     *
     * @experimental
     */
    addContainer(container: ContainerProps): Container;
    /**
     * (experimental) Add a volume to the pod.
     *
     * @experimental
     */
    addVolume(volume: Volume): void;
    /**
      * @internal
      */
    _toKube(): k8s.StatefulSetSpec;
}
